@@ -40,6 +40,7 @@
 #include "nm-setting-wireless.h"
 #include "nm-setting-wireless-security.h"
 #include "nm-setting-vpn.h"
+#include "nm-setting-olpc-mesh.h"
 
 #include "nm-setting-serial.h"
 #include "nm-setting-gsm.h"
@@ -215,6 +216,11 @@ register_default_settings (void)
 	                      NM_SETTING_WIRELESS_ERROR,
 	                      1);
 
+	register_one_setting (NM_SETTING_OLPC_MESH_SETTING_NAME,
+	                      NM_TYPE_SETTING_OLPC_MESH,
+	                      NM_SETTING_OLPC_MESH_ERROR,
+	                      1);
+
 	register_one_setting (NM_SETTING_GSM_SETTING_NAME,
 	                      NM_TYPE_SETTING_GSM,
 	                      NM_SETTING_GSM_ERROR,
@@ -1006,6 +1012,7 @@ nm_connection_new (void)
 /**
  * nm_connection_new_from_hash:
  * @hash: the #GHashTable describing the connection
+ * @error: location of a #GError to return on failure
  *
  * Creates a new #NMConnection from a hash table describing the connection.  See
  * nm_connection_to_hash() for a description of the expected hash table.
