@@ -32,7 +32,7 @@
 #include <linux/types.h>
 #include <linux/netlink.h>
 #include <linux/rtnetlink.h>
-#include <net/if.h>
+#include <linux/if.h>
 #include <linux/unistd.h>
 #include <unistd.h>
 #include <stdio.h>
@@ -206,14 +206,18 @@ netlink_object_message_handler (struct nl_object *obj, void *arg)
 
 	link_obj = (struct rtnl_link *) obj;
 	flags = rtnl_link_get_flags (link_obj);
-	if (flags & IFF_RUNNING) {
-        g_signal_emit (G_OBJECT (monitor), 
-                                signals[CARRIER_ON],
-                                0, rtnl_link_get_ifindex (link_obj));
+
+	/* IFF_LOWER_UP is the indicator of carrier status since kernel commit
+	 * b00055aacdb172c05067612278ba27265fcd05ce in 2.6.17.
+	 */
+	if (flags & IFF_LOWER_UP) {
+		g_signal_emit (G_OBJECT (monitor),
+		               signals[CARRIER_ON],
+		               0, rtnl_link_get_ifindex (link_obj));
 	} else {
-        g_signal_emit (G_OBJECT (monitor), 
-                                signals[CARRIER_OFF],
-                                0, rtnl_link_get_ifindex (link_obj));
+		g_signal_emit (G_OBJECT (monitor),
+		               signals[CARRIER_OFF],
+		               0, rtnl_link_get_ifindex (link_obj));
 	}
 
 out:
@@ -223,6 +227,11 @@ out:
 static int
 netlink_event_input (struct nl_msg *msg, void *arg)
 {
+	struct nlmsghdr *hdr = nlmsg_hdr (msg);
+
+	if (hdr->nlmsg_pid != 0)
+		return NL_STOP;
+
 	nl_msg_parse (msg, &netlink_object_message_handler, arg);
 
 	/* Stop processing messages */
@@ -395,8 +404,18 @@ deferred_emit_carrier_state (gpointer user_data)
 
 	priv->request_status_id = 0;
 
-	/* Emit each device's new state */
-	nl_cache_foreach_filter (priv->nlh_link_cache, NULL, netlink_object_message_handler, monitor);
+	/* Update the link cache with latest state, and if there are no errors
+	 * emit the link states for all the interfaces in the cache.
+	 */
+	if (nl_cache_refill (priv->nlh, priv->nlh_link_cache))
+		nm_warning ("error updating link cache: %s", nl_geterror ());
+	else {
+		nl_cache_foreach_filter (priv->nlh_link_cache,
+		                         NULL,
+		                         netlink_object_message_handler,
+		                         monitor);
+	}
+
 	return FALSE;
 }
 
@@ -411,15 +430,6 @@ nm_netlink_monitor_request_status (NMNetlinkMonitor  *monitor,
 	priv = NM_NETLINK_MONITOR_GET_PRIVATE (monitor);
 	g_return_val_if_fail (priv->event_id > 0, FALSE);
 
-	/* Update the link cache with latest state */
-	if (nl_cache_refill (priv->nlh, priv->nlh_link_cache)) {
-		g_set_error (error, NM_NETLINK_MONITOR_ERROR,
-		             NM_NETLINK_MONITOR_ERROR_LINK_CACHE_UPDATE,
-		             _("error updating link cache: %s"),
-		             nl_geterror ());
-		return FALSE;
-	}
-
 	/* Schedule the carrier state emission */
 	if (!priv->request_status_id)
 		priv->request_status_id = g_idle_add (deferred_emit_carrier_state, monitor);
